
# if somebody does "from somepackage import *", this is what they will
# be able to access:
__all__ = [
    'connect',
]


from pyrasgo.connection import RasgoConnection

def connect(api_key):
    conn = RasgoConnection(api_key)
    return conn





from pyrasgo.member import Member

class FeatureList(object):
    
    def __init__(self, api_object):
        self._api_object = api_object

    def join_keys(self):
        return self._api_object['join_keys']

    def id(self):
        return self._api_object['_id']

    def name(self):
        return self._api_object['name']

    def description(self):
        return self._api_object.get('description')

    def author(self):
        return Member(self._api_object.get('author'))

    def snowflake_table_metadata(self):
        features = self._api_object
        metadata = {
            "database": features.get("database", "rasgoalpha"),
            "schema": features.get("schema", "public"),
            "table": self._snowflake_table_name(),
            }
        return metadata

    def _snowflake_table_name(self):
        table_name = self._api_object.get("dataTableName")
        if table_name is None:
            raise ValueError("No Table found for Feature List '{}'".format(self.id()))
        return table_name

    def __repr__(self):
        return str(self)

    def __str__(self):
        return "FeatureList(id={}, name={}, description={}".format(self.id(), self.name(), self.description())



import flask
from flask import request, jsonify
import dill
import jsonpickle

app = flask.Flask(__name__)

@app.route('/verify')
def verify():
	return jsonify(success=request.args['key'] == 'bce2dad0-39dc-4840-9e47-3929cbf20cc0')

def verify_key(key):
	return key == 'bce2dad0-39dc-4840-9e47-3929cbf20cc0'

@app.route('/pull')
def pull():
	assert verify_key(request.args['key']), 'Bad API key.'
	assert 'feature_id' in request.args.to_dict(), 'No feature ID parameter included in request.' 
	return func_handler(request.args['feature_id'])

def func_handler(feature_id):
	if feature_id == 't1gnamzko8bco7795p1s':
		return jsonpickle.encode(dill.dumps(t1gnamzko8bco7795p1s))
	else:
		return 'DID NOT WORK'

def t1gnamzko8bco7795p1s(df):
	import pandas as pd
	hourly_temp_df = df[['time', 'icon']]
	hourly_temp_df['time'] = pd.to_datetime(hourly_temp_df['time'], unit='s').dt.strftime("%Y-%m-%d %H:%M")
	return hourly_temp_df

def id(name):
	if name == 'Jared':
		return 'CEO'
	elif name == 'Patrick':
		return 'CTO'
	elif name == 'Will':
		return 'SAIA Driver'

app.run(host='0.0.0.0', port=6500, ssl_context='adhoc', debug=True)



"""
This module contains all functions related to pulling the features/feature scripts.
A successful query results in data being stored where the user indicates (AWS/local).
"""

from pull import *

def get_feature(feature_ids: list) -> None:
	#pull_code(feature_ids)
	return pull_code(feature_ids)


# build out helper functions to allow for simple unit testing






import os
import requests
import amplitude
import pandas as pd
import snowflake.connector

from pyrasgo.feature_list import FeatureList

class RasgoConnection(object):

    def __init__(self, api_key):
        self._production_domain = "api.rasgoml.com"
        self._amplitude_key = "08836b1e33763119e01fc62c604a7ea8"
        self._user_id = None
        
        self._api_key = api_key
        self._hostname = os.environ.get("RAGSO_DOMAIN", self._production_domain)
        self._event_logger = amplitude.AmplitudeLogger(api_key = self._amplitude_key)
        if self._hostname != self._production_domain:
            self._event_logger.turn_off_logging()

    def get_lists(self):
        self.note({'event_type': 'get_lists'})
        response = self._request("/features/lists/")
        response.raise_for_status()
        lists = response.json()['data']
        return [FeatureList(entry) for entry in lists]

    def get_feature_list(self, list_id):
        self.note({'event_type': 'get_feature_list'})
        response = self._request("/features/lists/{}".format(list_id))
        response.raise_for_status()
        entry = response.json()['data']
        return FeatureList(entry)

    def get_feature_data(self, feature_list_id):
        self.note({'event_type': 'get_feature_data'})
        feature_list = self.get_feature_list(feature_list_id)
        
        conn = self._snowflake_connection(feature_list.author())
        
        table_metadata = feature_list.snowflake_table_metadata()
        query = self._make_select_statement(table_metadata)
        result_set = self._run_query(conn, query)
        return pd.DataFrame.from_records(iter(result_set), columns=[x[0] for x in result_set.description])

    def _run_query(self, conn, query):
        cursor = conn.cursor().execute(query)
        return cursor

    def _make_select_statement(self, table_metadata):
        return "SELECT * FROM {database}.{schema}.{table}".format(**table_metadata)
    
    def _snowflake_connection(self, member):
        creds = member.snowflake_creds()
        conn = snowflake.connector.connect(**creds)
        return conn

    def _request(self, resource):
        return requests.get(self._url(resource), headers=self._headers())

    def _headers(self):
        headers = {
            "accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer %s" % self._api_key,
            }
        return headers
    
    def _url(self, resource):
        return f'https://{self._hostname}/api/v1{resource}'
        
    def note(self, event_dict):
        event_dict['user_id'] = self.get_user_id()
        event_dict['source'] = 'pyrasgo'
        event = self._event_logger.create_event(**event_dict)
        self._event_logger.log_event(event)

    def get_user_id(self):
        if self._user_id:
            return self._user_id
        else:
            response = self._request("/members/me")
            response.raise_for_status()
            try:
                response_body = response.json()
                user = response_body['data']
                self._user_id = user.get('_id')
                return self._user_id
            except Exception:
                return None




import os

class Member(object):
    
    def __init__(self, api_object):
        self._api_object = api_object

    def snowflake_creds(self):
        supplied_creds = self._api_object.get("snowflake", {})

        if "user" not in supplied_creds and "username" in supplied_creds:
            # The Rasgo API supplies this value as "username", but the Snowflake API expects "user". 
            supplied_creds["user"] = supplied_creds["username"]
        
        default_creds = {
            "user": os.environ.get('SNOWFLAKE_USERNAME'),
            "password": os.environ.get('SNOWFLAKE_PASSWORD'),
            "account": "aya46528",
            "database": "RASGOALPHA",
            "schema":  "public",
            "warehouse": "COMPUTE_WH",
            }
        fields = ["account", "user", "password", "database", "schema", "warehouse", "role"]
        creds = dict([(f, supplied_creds.get(f, default_creds.get(f))) for f in fields])

        if creds["role"] is None:
            # If the Rasgo API does not supply a role, we infer it from their user name. We
            # need to do this after the fact because we need to determine what the user name
            # is before we can perform the inference.
            creds["role"] = creds["user"]  + "_role"
        return creds

        



"""
Internal module to help consolidate all code related 
to pulling encrypted feature engineering scripts.
"""

import yaml
import requests
import warnings
import dill
import jsonpickle

warnings.filterwarnings('ignore') # Keep getting InsecureRequestWarning because hosting API on local without certs

def pull_code(feature_ids: list) -> list:
	# Load configuration file containing API keys and endpoints
	parse_config()
	parse_endpoints()
	# Verify that the API key provided is valid
	verify_key()
	# Key verified
	# Need to verify key with every request
	feature_funcs = [get_feature(x) for x in feature_ids]
	return feature_funcs

def get_feature(feature_id: str) -> dict:
	feature_endpt = endpoints['rasgo_endpoints']['feature_pull']
	# The GET request should return a JSON that includes the feature UID, feature name, description, and jsonpickle object
	# The way we will enable users to actually use the feature scripts is as follows:
	# import jsonpickle
	# func = jsonpickle.decode(r.json()['func_header'])
	# func now has effectively replaced the function name and can be used with all its arguments i.e. func(a, b)
	r = requests.get(url=feature_endpt, params={'key': api_key, 'feature_id': feature_id}, verify=False)
	assert (r.status_code == 200), "Problem pulling feature. Please verify that the unique feature ID is correct."
	return dill.loads(jsonpickle.decode(r.content))

def verify_key():
	verify_endpt = endpoints['rasgo_endpoints']['verify']
	global api_key
	api_key = config['rasgo_credentials']['access_key']
	r = requests.get(url=verify_endpt, params={'key': api_key}, verify=False)
	assert (r.json()['success'] == True), "Problem validating API key. Please double check your API key." # Not a successful verification of API key
	print("\nSuccessfully validated API key.")

def parse_config():
	global config
	with open('..\\config\\config.yaml') as f:
		config = yaml.load(f)

def parse_endpoints():
	global endpoints
	with open('endpoints.yaml') as f:
		endpoints = yaml.load(f)


# EVERYTHING BELOW HERE IS FOR TESTING PURPOSES

def test_func_pull(feature_id):
	r = requests.get(url=endpoints['rasgo_endpoints']['feature_pull'], params={'key': api_key, 'feature_id': feature_id}, verify=False)
	func = dill.loads(jsonpickle.decode(r.content))
	return func

def dark_sky_sample():
	import pandas as pd
	secret = '4e79a3dd798e30beb50c84a3b37bd5a0'
	latitude = '43.142'
	longitude = '-85.049'
	time = '1550008800'
	url = f'https://api.darksky.net/forecast/{secret}/{latitude},{longitude},{time}'
	r = requests.get(url)
	df = pd.DataFrame(r.json()['hourly']['data'])
	return df

def run_test(feature_id):
	# Load configuration file containing API keys and endpoints
	parse_config()
	parse_endpoints()
	# Verify that the API key provided is valid
	verify_key()
	# Get base Dark Sky data
	# base_df = dark_sky_sample()
	# print('\n\nRaw DF from Dark Sky API')
	# print(base_df.head())
	# Retreive function
	func = test_func_pull(feature_id)
	return func
	# new_df = func(base_df)
	# print('\n\nFeature engineered DF after Dark Sky')
	# print(new_df.head())


# run_test()


from get import get_feature
import requests

# EVERYTHING BELOW FOR TESTING
def dark_sky_sample():
	import pandas as pd
	secret = '4e79a3dd798e30beb50c84a3b37bd5a0'
	latitude = '43.142'
	longitude = '-85.049'
	time = '1550008800'
	url = f'https://api.darksky.net/forecast/{secret}/{latitude},{longitude},{time}'
	r = requests.get(url)
	df = pd.DataFrame(r.json()['hourly']['data'])
	return df


base_df = dark_sky_sample()
print('\n\nRaw DF from Dark Sky API')
print(base_df.head())
func = get_feature(['t1gnamzko8bco7795p1s'])[0] #This is the only Rasgo interaction the user will have
new_df = func(base_df)
print('\n\nFeature engineered DF after Dark Sky')
print(new_df.head())
